/*
MiniShare - Minimal HTTP Server
Copyright (C) 2002-2004  Tero Lindeman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include "error.h"
#include "server.h"
#include "misc.h"
#include "minishare.h"
#include "template.h"

#define ERRORS 11

const error_message message[ERRORS]={
	{400,"Bad request",""},
	{401,"Unauthorized","WWW-Authenticate: Basic realm=\"MiniShare\"\r\n"},
	{403,"Forbidden",""},
	{404,"File not found",""},
	{405,"Method not allowed","Allow: GET, HEAD"},
	{413,"Request entity too large",""},
	{416,"Range not satisfiable",""},
	{500,"Internal server error",""},
	{501,"Not implemented",""},

	{503,"Service unavailable",""},
	{505,"HTTP version not supported",""}
};

char * get_error_message(int code,char *buffer)
{
	buffer[0]='\0';
	
	int i;
	for (i=0;i<ERRORS && message[i].code!=code;i++);
	if (i<ERRORS) {
		strcpy(buffer,message[i].message);
	} 
	
	return buffer;
}	

void send_error(SOCKET s,connection *c,int code) {
	char header[800],page[1000],err[100],smessage[100];
	int i;
	for (i=0;i<ERRORS && message[i].code!=code;i++);
	if (i<ERRORS) {
		strcpy(header,message[i].header);
		strcpy(smessage,message[i].message);
	} else {
		strcpy(header,"");
		strcpy(smessage,"");
	}
	
	sprintf(err,"%d %s",code,smessage);
	sprintf(page,"HTTP/1.1 %s\r\n%sContent-Type: text/html\r\n\r\n",err,header);
	/*send_sig(s,c);*/

	addtolog(c->con_id,err);
	
	char temp_path[MAX_PATH];
	PREPROCESSOR *preprocessor=open_preprocessor(make_workpath_file("layout\\error.tmpl",temp_path));
	    
    if (preprocessor!=NULL) {
        sprintf(page,"HTTP/1.1 %s\r\n%sContent-Type: text/html\r\n\r\n",err,header);
        sendmsg(s,page,c);
	/*send_sig(s,c);*/

        preprocessor->statuscode=code;
    	char temp_line[513];
    	int r;
    	do {
    		r=template_getline(temp_line,512,preprocessor);
    		sendmsg(s,temp_line,c);
	} while (r);
    	
    	close_preprocessor(preprocessor);
    }
    else {
		char temp_html[511];
		addtolog(c->con_id,"Error opening template file 'layout\\error.tmpl'");
		sprintf(temp_html,"<HTML><HEAD><TITLE>Error %s</TITLE></HEAD><BODY><P>Error %s</P><P>Error opening error template file</P></BODY></HTML>",err,err);
		sprintf(page,"HTTP/1.1 %s\r\nServer: MiniShare\r\nConnection: close\r\nPragma: no-cache\r\nCache-Control: no-cache\r\nContent-Type: text/html\r\nContent-Length: %d\r\n\r\n%s",err,strlen(temp_html),temp_html);
		sendmsg(s,page,c);
    }
	
}

