/*
MiniShare - Minimal HTTP Server
Copyright (C) 2002-2004  Tero Lindeman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "serve2.h"
#include "globals.h"
#include "files.h"
#include "error.h"
#include "misc.h"
#include <time.h>
#include <winsock2.h>
#include <windows.h>

typedef struct {
	int id;
	char *name;
} field_id;

#define HEADERFIELDS 4

static const field_id field[HEADERFIELDS]= {
	{REQ_HEAD_RANGE,"Range"},
	{REQ_HEAD_CONNECTION,"Connection"},
	{REQ_HEAD_AUTH,"Authorization"},
	{REQ_HEAD_REFERER,"Referer"}
};

int get_header_field(char *ln,char *value) {
	int i;
	char *tmp=(char*)malloc(MAXHEADERSIZE+2);
	sscanf(ln,"%200[^:]: %200[^\r\n]",tmp,value);
	for (i=0;i<HEADERFIELDS;i++) {
		//addtolog(0,"FIELD:")	;
		//char tt[800];
		//sprintf(tt,"'%s' '%s'",tmp,field[i].name);
		//addtolog(0,tt)	;
		if (strcmp(tmp,field[i].name)==0) {


			return field[i].id;
		}
	}
	free(tmp);
	return REQ_HEAD_NONE;
}

int send_response_header(connection *c,http_request *req) {
	char *header=(char*)malloc(MAXHEADERSIZE+2);



	if ((req->type&REQ_PARTIAL)!=0 && c->file!=-1) {
		sprintf(header,"HTTP/1.1 206 Partial Content\r\nContent-Range: bytes %lld-%lld/%lld\r\n",c->range_start,c->range_end,
			sharedfiles[c->file].filesize);
		sendmsg(c->socket,header,c);
		addtolog(c->con_id,"206 Partial Content");
	} else {
		sendmsg(c->socket,"HTTP/1.1 200 OK\r\n",c);
		addtolog(c->con_id,"200 OK");
	}

	char fd[1000];
	if (c->file!=-1) strcpy(fd,sharedfiles[c->file].filedate); else gettimestamp("layout\\minishare.css",fd);
	
	SYSTEMTIME sys_time;
	GetSystemTime(&sys_time);
	
	char date_now[50];
	systemtime_to_http_timestamp(&sys_time,date_now);
	
	char temp_mime[1000];
	if (c->file!=-1)
	{
		strcpy(temp_mime,sharedfiles[c->file].mime);
	}
	else
	{
		getmimetype("minishare.css",temp_mime);	
	}
	
	sprintf(header,
		"Date: %s\r\n"
		"Accept-Ranges: bytes\r\nContent-Length: %lld\r\nContent-Type: %s\r\nLast-Modified: %s\r\nConnection: %s\r\n\r\n",
		date_now,
		c->range_end-c->range_start+1,
		temp_mime,
		fd,
		req->keepalive==0?"close":"Keep-Alive");
		
	sendmsg(c->socket,header,c);
	free(header);
	return 1;
}

int send_file_rangeless(connection *c,char *fn){
	c->range_start=0;
	c->range_end=get_filesize(fn);
	return  send_file(c,fn);
}

int send_file(connection *c,char *fn){

	FILE *f;
	f=fopen(fn,"rb");
	if (f==NULL) {
		char tmp[strlen(fn)+100];
		sprintf(tmp,"Can't open '%s'.",fn);
		addtolog(c->con_id,tmp);
		return 0;
	}
	fseek(f,c->range_start,SEEK_SET);

	int finished=0,fpos=ftell(f);
	int touched=time(NULL);
	while (!finished) {
		if ((time(NULL)-touched)>30) 
		{
			addtolog(c->con_id,"Timeout while sending file.");
			goto quit;
		}
		char buffer[SEND_BUFFER_SIZE+2];
		int r=fread(buffer,1,SEND_BUFFER_SIZE,f);
		if (fpos+r>c->range_end)  {
			r=c->range_end-fpos+1;
		}
		if (r==0) finished=1; else {
			int sent;
			if ((sent=send(c->socket,buffer,r,0))==SOCKET_ERROR) finished=1;
			else {
				if (r!=sent) 
				{
					addtolog(c->con_id,"Transfer error.");
					finished=1;
				}
				c->bytesdownloaded+=sent;
				c->bytesdownloaded_file+=sent;
				c->bytesdownloaded_sec[TRANSFER_SPEED_BUFFER_SIZE-1]+=sent;
				touched=time(NULL);
			}
		}
	}
quit:
	fclose(f);
	return 1;
}


int get_request_header(SOCKET s,http_request *req,int con_id) {
	
	memset(req,0,sizeof(http_request));
	
	req->keepalive=0;
	char header[MAXHEADERSIZE+2];
	int hs=0,r=SOCKET_ERROR;
	int touched=time(NULL);
	do {
		r=recv(s,header+hs,MAXHEADERSIZE-hs,0);
		
		if ((time(NULL)-touched)>30) return 1;
		
		if (r==0 || r==WSAECONNRESET) return REQ_ERR_MALFORMED_REQUEST; 
		else 
		{
			if (r>0)
			{
				touched=time(NULL);
				hs+=r;
				if (hs>=MAXHEADERSIZE-2) return REQ_ERR_REQENTITYTOOLARGE;
				header[hs]='\0';
			}
		}
		Sleep(50); 	// So the thread doesn't eat all the CPU when waiting for header
					// Limits the transfer speed but should be OK as a header probably
					// fits in MAXHEADERSIZE (would require extra cycle to check if too big)
	} while (strcmp(header+hs-4,"\r\n\r\n")!=0 || r==SOCKET_ERROR);

	char *saveptr;
	char *request_line=strtok_r(header,"\r",&saveptr);
	if (request_line==NULL) return REQ_ERR_MALFORMED_REQUEST;
	request_line[1024]='\0';
	addtolog(con_id,request_line);
	
	req->request_string[0]='\0';

	char method[MAXHEADERSIZE+2],ver[MAXHEADERSIZE+2];
	if (sscanf(request_line,"%10s %1000[^?]? HTTP/%10s",method,req->path,ver)!=3)
		if (sscanf(request_line,"%10s %1000[^?]?%1000s HTTP/%10s",method,req->path,req->request_string,ver)!=4)
			if (sscanf(request_line,"%10s %1000s HTTP/%10s",method,req->path,ver)!=3) return REQ_ERR_MALFORMED_REQUEST;
		
	
	//if (strcmp(ver,"1.1")!=0) return REQ_ERR_VERSIONNOTSUPPORTED;
	// ^-- should be handled with a 101 page
	if (strcmp(method,"GET")!=0 && strcmp(method,"HEAD")!=0) return REQ_ERR_METHODNOTSUPPORTED;
	req->type=(strcmp(method,"GET")==0)?REQ_GET:REQ_HEAD;
	char *header_line[MAXHEADERLINES];
	char *tmp;
	int h=0;
	do {
		tmp=strtok_r(NULL,"\r",&saveptr);
		if (tmp!=NULL) {
			header_line[h]=tmp+1;
			//addtolog(0,header_line[h]);
			h++;
		}
	} while (tmp!=NULL&&h<MAXHEADERLINES);

	req->auth=0;
	req->keepalive=0;

	int i;
	for (i=0;i<h-2;i++) {
		char param[MAXHEADERSIZE+2];
		switch (get_header_field(header_line[i],param)) {
			case REQ_HEAD_RANGE:
				//addtolog(0,header_line[i]);
				req->type|=REQ_PARTIAL;
				if (sscanf(param,"bytes=%lld-%lld",&req->range_start,&req->range_end)==2) break;
				if (sscanf(param,"bytes=%lld-",&req->range_start)==1) {
					req->range_end=-1;
					break;
				}
				if (sscanf(param,"bytes=-%lld",&req->range_end)==1) {
					req->range_start=-1;
					break;
				}
			break;
			case REQ_HEAD_AUTH:
				//addtolog(666,param);
				if (sscanf(param,"Basic %s",req->auth_hash)!=1)
				{
					return REQ_ERR_MALFORMED_REQUEST;
				}
				req->auth=1;
			break;
			case REQ_HEAD_CONNECTION:
				if (strcmp("close",param)==0) req->keepalive=0;
				if (strcmp("Keep-Alive",param)==0) req->keepalive=1;
			break;
			case REQ_HEAD_REFERER:
				strcpy(req->referrer,param);
			break;
			case REQ_HEAD_NONE:
			break;
			default:
				return REQ_ERR_MALFORMED_REQUEST;
			break;
		}
	}
	return REQ_ERR_SUCCESS;
}

int sendcss(SOCKET s,connection *c,http_request *r)
{
	char temp_path[MAX_PATH];
	
	if (!file_exists(make_workpath_file("layout\\minishare.css",temp_path))) 
	{
		return 0;
	}
	
	r->range_start=0;
	r->range_end=get_filesize(make_workpath_file("layout\\minishare.css",temp_path))-1;
	
	if (r->range_end>=0)
	{
		r->type&=~REQ_PARTIAL;
		c->file=-1;
		c->range_start=r->range_start;
		c->range_end=r->range_end;
		
		send_response_header(c,r);
			
		if ((r->type&REQ_GET)!=0) 
		{
			send_file(c,make_workpath_file("layout\\minishare.css",temp_path));
		}
		return 1;	
	}
	else 
	{
		return 0;
	}
}

void _cdecl serve2(void* _cons) {
	SOCKET s=((connection*)_cons)->socket;
	connection *cons=((connection*)_cons);
	
	

	SendMessage(hWnd,WM_USER+6,0,0);
    
	http_request *req=(http_request*)malloc(sizeof(http_request));

again:  // <- sue me, g++ doesn't allow labels just before a "} while (blah);"

		// If the socket is non-blocking when the actual file trasfer starts, it will hang or
		// make the transfer _very_ slow. select() would be the right way but I'm way too lazy
		// to even think how it should be done with it. this seems work ok.
		
		// The reason why the socket has to be non-blocking is that the timeout will occur only
		// after something is received and the time has been over 30 seconds, now it will timeout
		// even if nothing has come thru yet (such as opening a telnet connection and typing nothing).
		
		block_socket(&s,1); 
		int err=get_request_header(s,req,cons->con_id);
		block_socket(&s,0);
		
		if (err==REQ_ERR_SUCCESS) {
			int f;
			strcpy(cons->request,req->path);
			if (strcmp(req->path,sharepage)==0)
			{
				sendlist(s,cons,req);
				goto finish;
			}
			if (strcmp(req->path,"/minishare.css")==0)
			{
				if (!sendcss(s,cons,req)) send_error(s,cons,404);
				goto finish;
			}
			
			f=findsharedfilebyurl(req->path);
			
			if ((f)==-1) 
			{
				send_error(s,cons,404); 
			} 
			else 
			{
				cons->file=f;	
				gettimestamp(sharedfiles[f].filename,sharedfiles[f].filedate);

				if ((sharedfiles[f].filesize=get_filesize(sharedfiles[f].filename))==-1) {
					send_error(s,cons,404);
					goto finish;
				}
				
				if (sharedfiles[f].auth!=-1) {
					if (!req->auth) {
						send_error(s,cons,401);
						goto finish;
					}
					if (!check_authority(sharedfiles[f].auth,req->auth_hash)) {
						send_error(s,cons,401);
						goto finish;
					}
				}
				
				if (sharedfiles[f].script)
				{
					sendpage(s,cons,req,sharedfiles[f].filename);
					goto finish;	
				}

				if ((req->type&REQ_PARTIAL)!=0) {
					if (req->range_start==-1) {
						req->range_start=sharedfiles[f].filesize-req->range_end;
						req->range_end=sharedfiles[f].filesize-1;
					} else if (req->range_end==-1) {
						req->range_end=sharedfiles[f].filesize-1;
					}

					if (req->range_start<0 || req->range_end>=sharedfiles[f].filesize || req->range_start>req->range_end) {
						send_error(s,cons,416);
						goto finish;
					}
				} else {
					req->range_start=0;
					req->range_end=sharedfiles[f].filesize-1;
				}
				
				cons->filesize=sharedfiles[f].filesize;
				cons->bytesdownloaded_file=0;
				cons->range_start=req->range_start;
				cons->range_end=req->range_end;
				send_response_header(cons,req);
				cons->bytesdownloaded_file=0;
				
				if ((req->type&REQ_GET)!=0) {
					send_file(cons,sharedfiles[cons->file].filename);
				}
			}

		} else {
			if (err!=1)
				send_error(s,cons,err);
			else
				addtolog(cons->con_id,"Timeout while waiting for request.");
		}

finish:
	if (req->keepalive && respect_keepalive) {
		addtolog(cons->con_id,"Reusing connection...");
		goto again;
	}

	shutdown(s,SD_BOTH);
    closesocket(s);
    connections--;

    char tmp[800];
    sprintf(tmp,"Total %lld bytes sent.",((connection*)_cons)->bytesdownloaded);
    addtolog(cons->con_id,tmp);

    addtolog(cons->con_id,"Connection closed.");
	((connection*)_cons)->inuse=false;

	SendMessage(hWnd,WM_USER+6,0,0);
	
	free(req);
}
